/*
// @HEADER
// ***********************************************************************
//
//          Tpetra: Templated Linear Algebra Services Package
//                 Copyright (2008) Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
//
// ************************************************************************
// @HEADER
*/

// Including this is the easy way to get access to all the Node types.
#include "Kokkos_DefaultNode.hpp"
#include "Tpetra_ConfigDefs.hpp"

// Don't bother compiling anything, or even including anything else,
// unless ThrustGPUNode is enabled.
#if defined(HAVE_TPETRA_EXPLICIT_INSTANTIATION) && defined(HAVE_KOKKOSCLASSIC_THRUST)

#include "Tpetra_CrsMatrixSolveOp_decl.hpp"
#include "TpetraCore_ETIHelperMacros.h"
#include "Tpetra_CrsMatrixSolveOp_def.hpp"

// FIXME (mfh 09 Jan 2015) This is unfortunately needed for
// CrsMatrix::localSolve().
#include "Tpetra_CrsMatrix_def.hpp"

// Instantiate only on float and double Scalar types for Thrust.  Only
// instantiate if Tpetra is supposed to instantiate for those Scalar
// types.

#if defined(HAVE_TPETRA_INST_FLOAT) && defined(HAVE_TPETRA_INST_DOUBLE)
// Instantiate for (float,double) and (double,float).
#  define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT( LO, GO )\
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT( float, double, LO, GO, KokkosClassic::ThrustGPUNode ) \
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT( double, float, LO, GO, KokkosClassic::ThrustGPUNode )
#else
#  define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT( LO, GO )
#endif // defined(HAVE_TPETRA_INST_FLOAT) && defined(HAVE_TPETRA_INST_DOUBLE)

#if defined(HAVE_TPETRA_INST_FLOAT)
#  if defined(HAVE_TPETRA_INST_DOUBLE)
// Instantiate for (float,float) and (double,double)
#    define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT_SINGLE( LO, GO )\
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT_SINGLE( float, LO, GO, KokkosClassic::ThrustGPUNode )\
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT_SINGLE( double, LO, GO, KokkosClassic::ThrustGPUNode )
#  else
// Instantiate for (float,float)
#    define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT_SINGLE( LO, GO )\
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT_SINGLE( float, LO, GO, KokkosClassic::ThrustGPUNode )
#  endif // defined(HAVE_TPETRA_INST_DOUBLE)
#elif defined(HAVE_TPETRA_INST_DOUBLE)
// Instantiate for (double,double)
#  define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT_SINGLE( LO, GO )\
  TPETRA_CRSMATRIX_SOLVEOP_INSTANT_SINGLE( double, LO, GO, KokkosClassic::ThrustGPUNode )
#else
// Don't instantiate at all.
#  define TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT_SINGLE( LO, GO )
#endif

namespace Tpetra {

  TPETRA_ETI_MANGLING_TYPEDEFS()

  TPETRA_INSTANTIATE_LG(TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT)
  TPETRA_INSTANTIATE_LG(TPETRA_CRSMATRIX_SOLVEOP_THRUSTGPUNODE_INSTANT_SINGLE)

} // namespace Tpetra

#endif // HAVE_TPETRA_EXPLICIT_INSTANTIATION && HAVE_KOKKOSCLASSIC_THRUST
